# -*- coding: utf-8 -*-
"""
Truck class
"""
from __future__ import absolute_import

import os.path

from PyQt5.QtGui import QPixmap, QFont, QPainter, QIcon, QColor, QBrush
from PyQt5.QtCore import QPointF, QRectF, pyqtSignal, QObject, QSettings
from PyQt5.QtWidgets import QMessageBox

from qgis.gui import QgsMapToolEmitPoint

from .simulator import CtrlCraftSimulator
from .settings import SIMU

settings = QSettings()


class CtrlCraftTruckLoadMapTool(QgsMapToolEmitPoint):
    """
    Map tool allowing to add a slab in a truck.
    """

    def __init__(self, truck):
        QgsMapToolEmitPoint.__init__(self, truck.master.canvas)
        self.truck = truck

    def canvasReleaseEvent(self, event):
        if self.truck.isFull():
            msg = self.tr('This truck is filled, you have to empty it first.')
            self.truck.master.iface.messageBar().pushWarning(self.tr('Warning'), msg)
        else:
            self.truck.addLoad(event.mapPoint())
            self.truck.master.refresh()

            if self.truck.isFull():
                msg = self.tr('The truck ')+'{}'.format(self.truck.id)+self.tr(' is filled.')
                self.truck.master.iface.messageBar().pushInfo(self.tr('Infos'), msg)


class CtrlCraftTruck(QObject):
    """
    Truck class
    """

    updated = pyqtSignal()

    def __init__(self, id, master):
        """
        Constructor.
        """
        QObject.__init__(self)
        self.id = id
        self.max_slab = self.getMaxSlab()
        self.master = master
        self.__actionLoad = None
        self.__actionUnload = None
        self.__actionState = None
        self.mapToolLoad = None

        self.init()

    @staticmethod
    def unloadLastSlab(master):
        if master.data.is_valid():
            id = master.data.truck_unload_last_slab()
            master.trucks[id].updateIconLoad()
            master.trucks[id].updateIconUnload()
            master.trucks[id].updateIconState()
            master.refresh()

    @property
    def loads(self):
        return self.master.data.truck_slabs(self.id)

    @property
    def actionLoad(self):
        return self.__actionLoad

    @actionLoad.setter
    def actionLoad(self, action):
        """ Associate action fill to truck """
        self.__actionLoad = action
        self.updateIconLoad()

    @property
    def actionUnload(self):
        return self.__actionUnload

    @actionUnload.setter
    def actionUnload(self, action):
        self.__actionUnload = action
        self.updateIconUnload()

    @property
    def actionState(self):
        return self.__actionState

    @actionState.setter
    def actionState(self, action):
        """ Associate action fill to truck """
        self.__actionState = action
        self.updateIconState()

    def activateMapTool(self):
        self.mapToolLoad = CtrlCraftTruckLoadMapTool(self)
        self.mapToolLoad.setAction(self.__actionLoad)
        self.master.canvas.setMapTool(self.mapToolLoad)

    def removeLoads(self):
        self.master.data.truck_unload_slabs(self.id)
        self.updateIconLoad()
        self.updateIconUnload()
        self.updateIconState()

        if SIMU:
            deposit = self.master.currentDeposit
            panel = self.master.currentPanel
            blast = self.master.currentBlast

            CtrlCraftSimulator().gate_create(deposit, panel, blast, self.id)

    def addLoad(self, point):
        """ Add a load in the truck """
        if self.loads < self.max_slab:
            if self.master.data.check_fixed_meaures_on_slabs(point) == 0 :
                text = self.tr("This slab has no fixed measure associated.\n Do you really want to load it without a fixed measure ?")
                dlg = QMessageBox(QMessageBox.Information, self.tr('No fixed measure'), text, QMessageBox.Yes | QMessageBox.No)
                return_value = dlg.exec()
                if return_value == QMessageBox.No :
                    return
            self.master.data.truck_load_slab(self.id, point)
            self.master.data.settings_probe_min_date_update()
            self.updateIconLoad()
            self.updateIconUnload()
            self.updateIconState()

        return self.isFull()

    def updateIconLoad(self):
        """ Update fill button icon """
        if not self.actionLoad:
            return

        name = self.loadIcon()
        path = os.path.join(os.path.dirname(__file__), 'img', name)
        pix = QPixmap(path)
        font = QFont()
        font.setPointSizeF(10)
        textPix = self.id

        position = QPointF(0, 60)
        painter = QPainter()
        painter.begin(pix)
        painter.setFont(font)
        painter.drawText(position, textPix)

        width = pix.width()
        if not self.isFull():
            width = (pix.width()/self.max_slab) * self.loads
        rect = QRectF(0.0, 0.0, width, pix.height())
        brush = QBrush(QColor(255, 0, 0, 150))
        painter.fillRect(rect, brush)

        painter.end()

        icon = QIcon(pix)

        self.__actionLoad.setIcon(icon)

        self.updated.emit()

    def updateIconUnload(self):
        """
        Update empty button icon
        """

        if not self.actionUnload:
            return

        iconName = self.unloadIcon()

        pix = QPixmap(os.path.join(os.path.dirname(__file__), 'img', iconName))
        font = QFont()
        font.setPointSizeF(10)
        textPix = self.id

        position = QPointF(0, 60)
        painter = QPainter()
        painter.begin(pix)
        painter.setFont(font)
        painter.drawText(position, textPix)
        painter.end()

        icon = QIcon(pix)

        self.actionUnload.setIcon(icon)

        self.actionUnload.setEnabled(not self.isEmpty())

    def updateIconState(self):
        """ Update fill button icon """
        if not self.actionState:
            return

        name = self.stateIcon()
        path = os.path.join(os.path.dirname(__file__), 'img', name)
        pix= QPixmap(path)
        font = QFont()
        font.setPointSizeF(10)
        probe = self.master.data.get_current_probe_radiometry(self.id)
        if probe :
            textPix = str(probe)[:5]
            probe= float(probe)

            rect = QRectF(0.0, 0.0, pix.width(), pix.height())
            if probe <= 0.6 :
                 brush = QBrush(QColor(140, 140, 140)) 
            elif probe > 0.6 and probe <= 0.9 :
                 brush = QBrush(QColor(222, 200, 0))
            elif probe > 0.9 and probe <= 1.2 :
                 brush = QBrush(QColor(0, 217, 0))
            elif probe > 1.2 and probe <= 1.8 :
                 brush = QBrush(QColor(206, 0, 0))
            elif probe > 1.8 and probe <= 3 :
                 brush = QBrush(QColor(0, 95, 213))
            elif probe > 3 :
                 brush = QBrush(QColor(142, 0, 215))
            painter = QPainter()
            painter.begin(pix)
            painter.fillRect(rect, brush)

            position = QPointF(0, 60)
            painter.setFont(font)
            painter.drawText(position, textPix)

            painter.end()
        else :
            textPix = 'ND'
            position = QPointF(0, 60)
            painter = QPainter()
            painter.begin(pix)
            painter.setFont(font)
            painter.drawText(position, textPix)
            rect = QRectF(0.0, 0.0, pix.width(), pix.height())
            brush = QBrush(QColor(255, 255, 255, 0))
            painter.fillRect(rect, brush)

            painter.end()

        icon = QIcon(pix)

        self.__actionState.setIcon(icon)

    def loadIcon(self):
        """ Returns the initial fill icon """
        return "icon_truck.png"

    def unloadIcon(self):
        """ Returns the empty icon """
        return "icon_truck_empty.png"

    def stateIcon(self):
        """ Returns the white icon """
        return "white_logo.png"

    def init(self):
        """ Init the state of the truck """
        self.updateIconLoad()
        self.updateIconUnload()
        self.updateIconState()

    def getMaxSlab(self):
        """ Retrieve slab capacity from params.ini file"""
        max_slab = 8
        for ind in range(1,9):
            if settings.value('CtrlCraft/Truck{}/id'.format(ind), ind)== str(self.id):
                max_slab = int(settings.value('CtrlCraft/Truck{}/sc'.format(ind), 8))
                break
        return max_slab

    def isFull(self):
        """ Check if the truck is full """
        return self.loads == self.max_slab

    def isEmpty(self):
        return self.loads == 0
